% =========================================================================
% Title       : Fast routine to compute l_inf proximal operator
% File        : prox_inf.m
% -------------------------------------------------------------------------
%
% Description:
%
%   Quite fast NlogN routine to compute the l_inf proximal operator:
%
%             xhat = argmin ||x||_inf + (1/2t)||x-w||^2
%
% Usage:
%
%   xhat = prox_inf(w,t)
%
%   A        : vector to be truncated
%   t        : step size
%
% ------------------------------------------------------------------------- 
%
% Revisions:
%
%   Date       Version  Author  Description
%   Jun-08-15  1.0      studer  cleanup for code release
%
% -------------------------------------------------------------------------
% (C) 2015 Christoph Studer (e-mail: studer@cornell.edu)
% =========================================================================

function [ xk ] = prox_inf( w,t )
    N = length(w);
    wabs = abs(w);
    ws = (cumsum(sort(wabs,'descend'))- t)./(1:N)';
    alphaopt = max(ws);
    if alphaopt>0 
      xk = min(wabs,alphaopt).*sign(w); % truncation step
    else
      xk = zeros(size(w)); % solution is zero otherwise
    end       
end